@@ -1,7 +1,7 @@
 /*
  *   BSD LICENSE
  *
- *   Copyright(c) 2016-2017 Intel Corporation. All rights reserved.
+ *   Copyright(c) 2016-2018 Intel Corporation. All rights reserved.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -66,8 +66,13 @@ rdt_cfg_print(FILE *stream, const struct rdt_cfg cfg)
 
 	switch (cfg.type) {
 	case PQOS_CAP_TYPE_L2CA:
-		fprintf(stream, "MASK: 0x%llx",
-			(unsigned long long) cfg.u.l2->ways_mask);
+		if (cfg.u.l2->cdp == 1)
+			fprintf(stream, "code MASK: 0x%llx, data MASK: 0x%llx",
+				(unsigned long long) cfg.u.l2->u.s.code_mask,
+				(unsigned long long) cfg.u.l2->u.s.data_mask);
+		else
+			fprintf(stream, "MASK: 0x%llx",
+				(unsigned long long) cfg.u.l2->u.ways_mask);
 		break;
 
 	case PQOS_CAP_TYPE_L3CA:
@@ -133,7 +138,14 @@ rdt_cfg_is_valid(const struct rdt_cfg cfg)
 
 	switch (cfg.type) {
 	case PQOS_CAP_TYPE_L2CA:
-		return cfg.u.l2 != NULL && cfg.u.l2->ways_mask != 0;
+		/* Validate L2 CAT configuration.
+		 *  - If CDP is enabled code_mask and data_mask must be set
+		 *  - If CDP is not enabled ways_mask must be set
+		 */
+		return cfg.u.l2 != NULL &&
+			((cfg.u.l2->cdp == 1 && cfg.u.l2->u.s.code_mask != 0 &&
+			cfg.u.l2->u.s.data_mask != 0) ||
+			(cfg.u.l2->cdp == 0 && cfg.u.l2->u.ways_mask != 0));
 
 	case PQOS_CAP_TYPE_L3CA:
 		/* Validate L3 CAT configuration.
@@ -372,16 +384,20 @@ rdt_ca_str_to_cbm(const char *param, struct rdt_cfg ca)
 	if (ret < 0)
 		return -EINVAL;
 
-	if (0 == mask || is_contiguous(rdt_cfg_get_type_str(ca), mask) == 0)
+	if (mask == 0 || is_contiguous(rdt_cfg_get_type_str(ca), mask) == 0)
+		return -EINVAL;
+
+	if (mask2 != 0 && is_contiguous(rdt_cfg_get_type_str(ca), mask2) == 0)
 		return -EINVAL;
 
 	if (PQOS_CAP_TYPE_L2CA == ca.type) {
-		if (mask2 != 0)
-			return -EINVAL;
-		ca.u.l2->ways_mask = mask;
+		if (mask2 != 0) {
+			ca.u.l2->cdp = 1;
+			ca.u.l2->u.s.data_mask = mask;
+			ca.u.l2->u.s.code_mask = mask2;
+		} else
+			ca.u.l2->u.ways_mask = mask;
 	} else {
-		if (mask2 != 0 && is_contiguous("L3", mask2) == 0)
-			return -EINVAL;
 		if (mask2 != 0) {
 			ca.u.l3->cdp = 1;
 			ca.u.l3->u.s.data_mask = mask;
@@ -769,8 +785,8 @@ get_contention_mask(const enum pqos_cap_type type)
 /**
  * @brief Returns cumulative mask for \a ca CAT config
  *
- * For L3 CDP config returns code_mask | data_mask,
- * for L2 or L3 non-CDP config returns ways_mask
+ * For L2/L3 CDP config returns code_mask | data_mask,
+ * for L2/L3 non-CDP config returns ways_mask
  *
  * @param [in] ca CAT COS configuration
  *
@@ -784,8 +800,13 @@ rdt_ca_get_cumulative_cbm(const struct rdt_cfg ca)
             NULL == ca.u.generic_ptr || 0 == rdt_cfg_is_valid(ca))
 		return UINT64_MAX;
 
-	if (PQOS_CAP_TYPE_L2CA == ca.type)
-		return ca.u.l2->ways_mask;
+	if (PQOS_CAP_TYPE_L2CA == ca.type) {
+		if (ca.u.l2->cdp == 1)
+			return ca.u.l2->u.s.code_mask | ca.u.l2->u.s.data_mask;
+		else
+			return ca.u.l2->u.ways_mask;
+	}
+
 	if (ca.u.l3->cdp == 1)
 		return ca.u.l3->u.s.code_mask | ca.u.l3->u.s.data_mask;
 
@@ -1061,9 +1082,17 @@ alloc_get_default_cos(struct pqos_l2ca *l2_def, struct pqos_l3ca *l3_def,
 		return -EFAULT;
 
 	if (m_cap_l2ca != NULL && l2_def != NULL) {
-		memset(l2_def, 0, sizeof(*l2_def));
-		l2_def->ways_mask =
+		const uint64_t def_mask =
 			(1ULL << m_cap_l2ca->u.l2ca->num_ways) - 1ULL;
+
+		memset(l2_def, 0, sizeof(*l2_def));
+
+		if (m_cap_l2ca->u.l2ca->cdp_on == 1) {
+			l2_def->cdp = 1;
+			l2_def->u.s.code_mask = def_mask;
+			l2_def->u.s.data_mask = def_mask;
+		} else
+			l2_def->u.ways_mask = def_mask;
 	}
 
 	if (m_cap_l3ca != NULL  && l3_def != NULL) {
